@@ -1,20 +1,22 @@
+import logging
+import multiprocessing
+import os
 import queue
 import socket
-import time
-import os
-from io import BytesIO
 import threading
-import multiprocessing
-import logging
-from ssl import SSLContext
+import time
+import traceback
 from dataclasses import dataclass, field
 from datetime import datetime, timezone
-from typing import Union, Dict, Callable, List, Optional, Tuple, Generator, Iterable
+from io import BytesIO
+from ssl import SSLContext
+from typing import Union, Callable, Optional, Generator, Iterable
 
 from . import utility
 
 
 class HookedSocket(socket.socket):
+    """An operateble socket class instead of native socket"""
 
     def __init__(self, sock: socket.socket, buffer: int = 1024):
         fd = sock.detach()
@@ -115,13 +117,13 @@ class HookedSocket(socket.socket):
 @dataclass
 class Request:
     """Request that recevived from the parser of server"""
-    addr: Tuple[str, int] = field(default = ('127.0.0.1', -1))
+    addr: tuple[str, int] = field(default = ('127.0.0.1', -1))
     method: str = field(default = 'GET')
     url: str = field(default = '/')
     version: str = field(default = 'HTTP/1.1')
-    keyword: Dict[str, str] = field(default_factory = dict)
+    keyword: dict[str, str] = field(default_factory = dict)
     arg: set = field(default_factory = set)
-    header: Dict[str, str] = field(default_factory = dict)
+    header: dict[str, str] = field(default_factory = dict)
     query: str = field(default = '')
     conn: socket.socket = None
     path: str = None
@@ -140,7 +142,7 @@ class Request:
     def iter_content(self, buffer: int = 1024) -> Generator[bytes, None, None]:
         """
         Get the content of the Request.\n
-        :param buffer: buffer size of content (if is_iter is True, or it will be ingored)
+        :param buffer: buffer size of content
         """
         content = True
         while content:
@@ -157,14 +159,16 @@ class Request:
         return (line + header + '\r\n\r\n').encode()
 
     def get_overview(self) -> str:
-        ht = '\n'.join(': '.join((h, self.header[h])) for h in self.header)
-        pr = '\n'.join(f'{k[0]}: {k[1]}' for k in self.keyword.items()) + '\n' + '; '.join(self.arg)
+        """Return a human-readable information of request"""
+        ht = '\n    '.join(': '.join((h, self.header[h])) for h in self.header)
+        pr = ' '.join(f'{k[0]}:{k[1]}' for k in self.keyword.items())
         return f'<Request [{self.url}]> from {self.addr} {self.version}\n' \
                f'Method: {self.method}\n' \
                f'Headers: \n' \
                f'{ht}\n' \
-               f'Params:' \
-               f'{pr}'
+               f'Query:{self.query}\n' \
+               f'Params:{pr}\n' \
+               f'Args:{" ".join(self.arg)}\n'
 
     def __repr__(self) -> str:
         return f'Request[{self.method} -> {self.url}]'
@@ -176,7 +180,7 @@ class Response:
     code: int = 200
     desc: str = None
     version: str = field(default = 'HTTP/1.1')
-    header: Dict[str, str] = field(default_factory = dict)
+    header: dict[str, str] = field(default_factory = dict)
     timestamp: Union[datetime, int] = int(time.time())
     content: Union[bytes, str] = b''
     encoding: str = 'utf-8'
@@ -194,7 +198,7 @@ class Response:
         self.header = {
                           'Content-Type': 'text/plain',
                           'Content-Length': str(len(self.content)),
-                          'Server': 'Lighting',
+                          'Server': 'Lightning',
                           'Date': self.timestamp.strftime('%a, %d %b %Y %H:%M:%S GMT')
                       } | self.header
         # Reset the content-type in header
@@ -212,7 +216,7 @@ class Response:
         return f'Response[{self.code}]'
 
     def __bool__(self) -> bool:
-        return self.code != 200 or self.content is not None
+        return self.code != 200 or self.content != b''
 
 
 InterfaceFunction = Callable[[Request], Optional[Union[Response, str]]]
@@ -222,11 +226,11 @@ class Interface:
     """The main HTTP server handler."""
 
     def __init__(self, func: InterfaceFunction, default_resp: Response = Response(code = 500),
-                 pre: List[Callable[[Request], Union[Request, Response, str, None]]] = None,
-                 post: List[Callable[[Request, Response], Response]] = None, strict: bool = False):
+                 pre: list[Callable[[Request], Union[Request, Response, str, None]]] = None,
+                 post: list[Callable[[Request, Response], Response]] = None, strict: bool = False):
         """
         :param func: A function to handle requests
-        :param default_resp: HTTP content will be sent when the function meets expections
+        :param default_resp: the HTTP content be sent when the exception occoured
         :param strict: if it is True, the interface will catch exceed path in interfaces and return a 404 response
         :param pre: things to do before the function processes request, the result will cover the function
         :param post: things to do post the function processed request, they should return a Response object
@@ -242,7 +246,7 @@ class Interface:
 
     def process(self, request: Request) -> Response:
         """
-        Let the function processes the request and returns the result
+        Let the function processes the request and returns the result\n
         :param request:  the request that will be processed
         """
         if self.strict and (request.path != '/'):
@@ -263,14 +267,30 @@ class Interface:
         return resp
 
     def __exit__(self, exc_type, exc_val, exc_tb):
-        return False
+        if exc_tb:
+            traceback.print_exception(exc_type, value = exc_val, tb = exc_tb)
+        return True
 
     def __repr__(self) -> str:
         return f'Interface[{self._function.__name__}]'
 
 
+class StaticInterface(Interface):
+    """An interface class that always returns a static response."""
+
+    def __init__(self, static_response: Optional[Response] = None, **kwargs):
+        self.response = Response(**kwargs) if static_response is None else static_response
+        super().__init__(lambda _: self.response)
+
+    def __repr__(self) -> str:
+        return f'StaticInterface[{self.response}]'
+
+
 class WSGIInterface(Interface):
     def __init__(self, application: Callable[[dict, Callable], Optional[Iterable[bytes]]], *args, **kwargs):
+        """
+        :param application: a callble object to run as a WSGI application
+        """
         # remove 'pre' and 'post' arguments, they are unusable in WSGI standards.
         self.app = application
         self.response: Response = Response()
@@ -295,7 +315,13 @@ class WSGIInterface(Interface):
             self.content_iter = ()
         return Response()  # Avoid interface to send response data
 
-    def start_response(self, status: str, header: List[Tuple[str, str]] = None, exc_info: tuple = None):
+    def start_response(self, status: str, header: list[tuple[str, str]] = None, exc_info: tuple = None):
+        """
+        Submit an HTTP response header, it will not be sent immediately.\n
+        :param status: HTTP status code and its description
+        :param header: HTTP headers like [(key1, value1), (key2, value2)]
+        :param exc_info: if an error occured, it is an exception tuple, otherwise it is None
+        """
         logging.info(f'start_response is called by {self.app.__name__}')
         if exc_info:
             raise exc_info[1].with_traceback(exc_info[2])
@@ -304,14 +330,15 @@ class WSGIInterface(Interface):
 
     @staticmethod
     def get_environ(request: Request) -> dict:
+        """Return a dict includes WSGI varibles from a request"""
         environ = {}
         environ.update(dict(os.environ))
         environ['REQUEST_METHOD'] = request.method.upper()
         environ['SCRIPT_NAME'] = request.url.removesuffix(request.path)
         environ['PATH_INFO'] = request.path.removeprefix('/')
         environ['QUERY_STRING'] = request.query
-        environ['CONTENT_TYPE'] = request.header.get('content-type') or ''
-        environ['CONTENT_LENGTH'] = request.header.get('content-length') or ''
+        environ['CONTENT_TYPE'] = request.header.get('Content-Type') or ''
+        environ['CONTENT_LENGTH'] = request.header.get('Content-Length') or ''
         environ['SERVER_NAME'], environ['SERVER_PORT'] = request.conn.getsockname()
         environ['SERVER_PROTOCOL'] = request.version
         map(lambda h: environ.update({f'HTTP_{h[0].upper()}': h[1]}), request.header.items())  # Set HTTP_xxx variables
@@ -333,12 +360,20 @@ class MethodInterface(Interface):
     def __init__(self, get: InterfaceFunction = None, head: InterfaceFunction = None, post: InterfaceFunction = None,
                  put: InterfaceFunction = None, delete: InterfaceFunction = None, connect: InterfaceFunction = None,
                  options: InterfaceFunction = None, trace: InterfaceFunction = None, patch: InterfaceFunction = None,
-                 strict: bool = True, *args, **kwargs):
+                 limited: bool = True, *args, **kwargs):
+        """
+        :param limited: if it is True, there is no default values for "OPTION" and "HEAD" method
+         otherwise the default values are present
+        """
+        if not limited:
+            options = options or self.options_
+            head = head or self.head_
         self.methods = {'GET': get, 'HEAD': head, 'POST': post, 'PUT': put, 'DELETE': delete, 'CONNECT': connect,
-                        'OPTIONS': options or (self.options_ if strict else options), 'TRACE': trace, 'PATCH': patch}
+                        'OPTIONS': options, 'TRACE': trace, 'PATCH': patch}
         super().__init__(func = self.select, *args, **kwargs)
 
     def select(self, request: Request) -> Optional[Response]:
+        """Return a response which is produced by specified method in request"""
         if request.method in self.methods:
             method = self.methods.get(request.method)
             if method:
@@ -358,6 +393,16 @@ class MethodInterface(Interface):
             resp.header.update({'Access-Control-Allow-Origin': request.header['Origin']})
         return resp
 
+    def head_(self, request: Request) -> Response:
+        """Default "HEAD" method implementation"""
+        get = self.methods['get']
+        if get:
+            resp = get(request)
+            resp.content = b''
+        else:
+            resp = Response(405)
+        return resp
+
     def __repr__(self) -> str:
         desc = "|".join(f"{m[0]}=>{m[1].__name__}" for m in self.methods.items() if m[1])
         return f'Interface[{utility.shrink_string(desc)}]'
@@ -374,7 +419,7 @@ DefaultInterface = Interface(default)
 class Node(Interface):
     """An interface that provides a main-interface to carry sub-Interfaces."""
 
-    def __init__(self, interface_map: Union[Callable[[], Dict[str, Interface]], Dict[str, Interface]] = None,
+    def __init__(self, interface_map: Union[Callable[[], dict[str, Interface]], dict[str, Interface]] = None,
                  default_interface: Interface = DefaultInterface, default_resp: Response = Response(code = 500)):
         """
         :param interface_map: Initral mapping for interfaces
@@ -383,7 +428,7 @@ class Node(Interface):
         """
         super().__init__(self.select, default_resp)
         self._map = interface_map or {}
-        self._tree: Dict[str, Interface] = {}
+        self._tree: dict[str, Interface] = {}
         self.default_interface = default_interface
         self.recent_repr: str = ''
 
@@ -405,17 +450,17 @@ class Node(Interface):
         return target_interface.process(request)
 
     @property
-    def map(self) -> Dict[str, Interface]:
+    def map(self) -> dict[str, Interface]:
         """Return interface map as a dictonary"""
         return {**(self._map() if isinstance(self._map, Callable) else self._map), **self._tree}
 
-    def bind(self, pattern: str, interface_or_method: Union[Interface, List[str]] = None, *args, **kwargs):
+    def bind(self, pattern: str, interface_or_method: Union[Interface, list[str]] = None, *args, **kwargs):
         r"""
         Bind an interface or function into this node.
 
         :param pattern: the url prefix that used to match requests.
-        :param interface_or_method: If the function has been using as a normal function, the value is the Interface
-            needs to bind. If the function has been using as a decorator or the value is None, the value is expected
+        :param interface_or_method: If the function be used as a normal function, the value is the Interface
+            needs to bind. If the function be used as a decorator or the value is None, the value is expected
             HTTP methods list. If the value is None, it means the Interface has no limits on methods.
         """
         if not pattern.startswith('/'):
@@ -437,6 +482,7 @@ class Node(Interface):
             return decorator
 
     def _repr(self) -> str:
+        """Return the desciption of itself (this method should never be called)"""
         desc = "|".join(f"{x[0]}=>{x[1]}" for x in self.map.items())
         return f'Node[{utility.shrink_string(desc)}]'
 
@@ -459,7 +505,7 @@ class Session:
         with self.interface as i:
             resp = i.process(self.request)
             logging.info(f'{i} processed successful with {resp}')
-            if resp:
+            if resp and not getattr(self.request.conn, '_closed'):
                 self.request.conn.sendall(resp.generate())
             self.request.conn.close()
         try:
@@ -511,5 +557,5 @@ class ProcessWorker(Worker, multiprocessing.Process):
         super().__init__(request_queue = request_queue, timeout = timeout)
 
 
-__all__ = ['Request', 'Response', 'Interface', 'MethodInterface', 'Node', 'HookedSocket', 'Session', 'Worker',
-           'ThreadWorker', 'ProcessWorker', 'DefaultInterface', 'WSGIInterface']
+__all__ = ['Request', 'Response', 'Interface', 'StaticInterface', 'MethodInterface', 'Node', 'HookedSocket', 'Session',
+           'Worker', 'ThreadWorker', 'ProcessWorker', 'DefaultInterface', 'WSGIInterface']
